#ifndef __WS2812_H__
#define __WS2812_H__

#include <stdint.h>

// parameter structure
typedef struct {
    uint32_t pin;
    uint32_t *colors;
    volatile uint32_t count;
} ws2812_params;

// driver header structure
typedef struct {
    uint32_t    jmp_inst;
    uint32_t    resettix;
    uint32_t    bit0hi;
    uint32_t    bit0lo;
    uint32_t    bit1hi;
    uint32_t    bit1lo;
    uint32_t    swaprg;
} ws2812_hdr;

#define SWAP_NONE   0   // use with the ws2811
#define SWAP_RG     1   // use with the ws2812

#define COLOR(r, g, b)      (((r) << 16) | ((g) << 8) | (b))
#define SCALE(x, l)         ((x) * (l) / 255)
#define COLORX(r, g, b, l)  ((SCALE(r, l) << 16) | (SCALE(g, l) << 8) | SCALE(b, l))

#define COLOR_BLACK     COLOR(  0,   0,   0)
#define COLOR_WHITE     COLOR(255, 255, 255)
#define COLOR_RED       COLOR(255,   0,   0)
#define COLOR_GREEN     COLOR(  0,   255, 0)
#define COLOR_BLUE      COLOR(  0,   0, 255)
#define COLOR_PINK		COLOR(0,  255,  255)
#define COLOR_ORANGE    COLOR(255,  0,  255)


int ws2812_init(int pin, uint32_t *colors, int count, int swaprg);
void ws2812_clock_init(void);
void ws2812_fill(uint32_t *colors, int count, uint32_t color);
void ws2812_fill_wheel(uint32_t *colors, int count, int delay);
void ws2812_fill_wheel_continuous(uint32_t *colors, int count, int samples, int delay);
void ws2812_fill_wheel_continuous_dim(uint32_t *colors, int count, int samples, int brightness, int delay);
void ws2812_fill_wheel_dim(uint32_t *colors, int count, int delay, int brightness);
uint32_t ws2812_wheel(int pos);
uint32_t ws2812_wheel_dim(int pos, int brightness);
void ws2812_pause(int ms);

#endif
